/****************************************************************************
** $Id: qg_filedialog.cpp,v 1.3 2003/09/11 19:19:24 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "qg_filedialog.h"

#include <qfiledialog.h>
#include <qmessagebox.h>

#include "rs_string.h"
#include "rs_settings.h"
#include "rs_system.h"


/**
 * Shows a dialog for choosing a file name. Saving the file is up to
 * the caller.
 *
 * @return File name with path and extension to determine the file type
 *         or an empty string if the dialog was cancelled.
 */
RS_String QG_FileDialog::getSaveFileName(QWidget* parent) {
    // read default settings:
    RS_SETTINGS->beginGroup("/Paths");
    RS_String defDir = RS_SETTINGS->readEntry("/Save",
                       RS_SYSTEM->getHomeDir());
    RS_String defFilter = RS_SETTINGS->readEntry("/SaveFilter",
                          "Drawing Exchange (*.dxf)");
    RS_SETTINGS->endGroup();

    // prepare file save as dialog:
    QFileDialog fileDlg(parent,0,true);
    QStringList filters;
    bool done = false;
    bool cancel = false;
    RS_String fn = "";

    filters.append("Drawing Exchange (*.dxf)");
    filters.append("Font (*.cxf)");
    fileDlg.setFilters(filters);
    fileDlg.setMode(QFileDialog::AnyFile);
    fileDlg.setCaption(QObject::tr("Save Drawing As"));
    fileDlg.setDir(defDir);
    fileDlg.setSelectedFilter(defFilter);

    // run dialog:
    do {
        // accepted:
        if (fileDlg.exec()==QDialog::Accepted) {
            fn = fileDlg.selectedFile();
            cancel = false;

            // append default extension:
            if (fn.find('.')==-1) {
                if (fileDlg.selectedFilter()=="Font (*.cxf)") {
                    fn+=".cxf";
                } else {
                    fn+=".dxf";
                }
            }

            // overwrite warning:
            if(QFileInfo(fn).exists()) {
                int choice =
                    QMessageBox::warning(parent, QObject::tr("Save Drawing As"),
                                         QObject::tr("%1 already exists.\n"
                                                     "Do you want to replace it?")
                                         .arg(fn),
                                         QObject::tr("Yes"), QObject::tr("No"),
                                         QObject::tr("Cancel"), 0, 1 );

                switch (choice) {
                case 0:
                    done = true;
                    break;
                case 1:
                case 2:
                default:
                    done = false;
                    break;
                }
            } else {
                done = true;
            }
        } else {
            done = true;
            cancel = true;
            fn = "";
        }
    } while(!done);

    // store new default settings:
    if (!cancel) {
        RS_SETTINGS->beginGroup("/Paths");
        RS_SETTINGS->writeEntry("/Save", QFileInfo(fn).dirPath(true));
        RS_SETTINGS->writeEntry("/SaveFilter", fileDlg.selectedFilter());
        RS_SETTINGS->endGroup();
    }

    return fn;
}



/**
 * Shows a dialog for choosing a file name. Opening the file is up to
 * the caller.
 *
 * @return File name with path and extension to determine the file type
 *         or an empty string if the dialog was cancelled.
 */
RS_String QG_FileDialog::getOpenFileName(QWidget* parent, RS2::FormatType* type) {
    // read default settings:
    RS_SETTINGS->beginGroup("/Paths");
    RS_String defDir = RS_SETTINGS->readEntry("/Open",
                       RS_SYSTEM->getHomeDir());
    RS_String defFilter = RS_SETTINGS->readEntry("/OpenFilter",
                          "Drawing Exchange (*.dxf *.DXF)");
    RS_SETTINGS->endGroup();

    RS_String fDxf(QObject::tr("Drawing Exchange %1").arg("(*.dxf *.DXF)"));
    RS_String fDxf1(QObject::tr("QCad 1.x file %1").arg("(*.dxf *.DXF)"));
    RS_String fCxf(QObject::tr("Font %1").arg("(*.cxf)"));

    RS_String fn = "";
    bool cancel = false;

    QFileDialog fileDlg(parent, "", true);

    QStringList filters;
    filters.append(fDxf);
    filters.append(fDxf1);
    filters.append(fCxf);

    fileDlg.setFilters(filters);
    fileDlg.setMode(QFileDialog::ExistingFile);
    fileDlg.setCaption(QObject::tr("Open Drawing"));
    fileDlg.setDir(defDir);
    fileDlg.setSelectedFilter(defFilter);

    if (fileDlg.exec()==QDialog::Accepted) {
        fn = fileDlg.selectedFile();
        if (type!=NULL) {
            if (fileDlg.selectedFilter()==fDxf1) {
                *type = RS2::FormatDXF1;
            } else if (fileDlg.selectedFilter()==fDxf) {
                *type = RS2::FormatDXF;
            } else if (fileDlg.selectedFilter()==fCxf) {
                *type = RS2::FormatCXF;
            }
        }
        cancel = false;
    } else {
        cancel = true;
    }

    // store new default settings:
    if (!cancel) {
        RS_SETTINGS->beginGroup("/Paths");
        RS_SETTINGS->writeEntry("/Open", QFileInfo(fn).dirPath(true));
        RS_SETTINGS->writeEntry("/OpenFilter", fileDlg.selectedFilter());
        RS_SETTINGS->endGroup();
    }

    return fn;
}

// EOF

