/****************************************************************************
** $Id: rs_system.cpp,v 1.8 2003/09/18 15:14:16 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "qapplication.h"
#include "rs_settings.h"
#include "rs_system.h"
#include "rs_regexp.h"
#include "rs_translator.h"
#include "rs_fileinfo.h"

RS_System* RS_System::uniqueInstance = NULL;


/**
 * Initializes the system.
 *
 * @param appName Application name (e.g. "QCad II")
 * @param appVersion Application version (e.g. "1.2.3")
 * @param appDirName Application directory name used for 
 *     subdirectories in /usr, /etc ~/.
 */
void RS_System::init(const RS_String& appName, const RS_String& appVersion,
                     const RS_String& appDirName) {
    this->appName = appName;
    this->appVersion = appVersion;
    this->appDirName = appDirName;
    this->appDir = RS_Dir::currentDirPath();

    RS_DEBUG->print("System %s initialized.", appName.latin1());
    RS_DEBUG->print("Current dir: %s.", appDir.latin1());
    initialized = true;

    initLanguageList();
}



/**
 * Initializes the list of available translations.
 */
void RS_System::initLanguageList() {
    RS_StringList lst = getFileList("qm", "qm");
		
    RS_SETTINGS->beginGroup("/Paths");
    lst += RS_StringList::split(";", 
			RS_SETTINGS->readEntry("/Translations", ""));
    RS_SETTINGS->endGroup();

    for (RS_StringList::Iterator it = lst.begin();
            it!=lst.end();
            ++it) {

        int i1 = (*it).find('_');
        int i2 = (*it).find('.', i1);
        RS_String l = (*it).mid(i1+1, i2-i1-1);

        if (languageList.find(l)==languageList.end()) {
            languageList.append(l);
        }
    }
}



/**
 * Loads a different translation for the application GUI.
 */
void RS_System::loadTranslation(const RS_String& lang, const RS_String& langCmd) {
    static RS_Translator* tQCad = NULL;
    static RS_Translator* tQCadGuiQt = NULL;
    static RS_Translator* tQCadActions = NULL;
    static RS_Translator* tQCadCmd = NULL;
    //static RS_Translator* tQCadLib = NULL;

    RS_String langFile;

    langFile = "qcad_" + lang + ".qm";
    if (tQCad!=NULL) {
        qApp->removeTranslator(tQCad);
        delete tQCad;
    }
    tQCad = new QTranslator(0);
    tQCad->load(langFile, "./qm");
    qApp->installTranslator(tQCad);


    langFile = "qcadguiqt_" + lang + ".qm";
    if (tQCadGuiQt!=NULL) {
        qApp->removeTranslator(tQCadGuiQt);
        delete tQCadGuiQt;
    }
    tQCadGuiQt = new QTranslator(0);
    tQCadGuiQt->load(langFile, "./qm");
    qApp->installTranslator(tQCadGuiQt);
	

    langFile = "qcadactions_" + lang + ".qm";
    if (tQCadActions!=NULL) {
        qApp->removeTranslator(tQCadActions);
        delete tQCadActions;
    }
    tQCadActions = new QTranslator(0);
    tQCadActions->load(langFile, "./qm");
    qApp->installTranslator(tQCadActions);

    langFile = "qcadcmd_" + langCmd + ".qm";
    if (tQCadCmd!=NULL) {
        qApp->removeTranslator(tQCadCmd);
        delete tQCadCmd;
    }
    tQCadCmd = new QTranslator(0);
    tQCadCmd->load(langFile, "./qm");
    qApp->installTranslator(tQCadCmd);
	
    /*
	langFile = "qcadlib_" + lang + ".qm";
    if (tQCadLib!=NULL) {
        qApp->removeTranslator(tQCadLib);
        delete tQCadLib;
    }
    tQCadLib = new QTranslator(0);
    tQCadLib->load(langFile, "./qm");
    qApp->installTranslator(tQCadLib);
	*/
}



/**
 * Checks if the system has been initialized and prints a warning
 * otherwise to stderr.
 */
bool RS_System::checkInit() {
    if (!initialized) {
        std::cerr << "Warning: System not initialized.\n"
        << "Use RS_SYSTEM->init(appname, appdirname) to do so.\n";
    }
    return initialized;
}



/**
 * Searches for files in an application shared directory in the given
 * subdirectory with the given extension.
 *
 * @return List of the absolute paths of the files found.
 */
RS_StringList RS_System::getFileList(const RS_String& subDirectory,
                                     const RS_String& fileExtension) {

    checkInit();

    /*RS_StringList dirList;

    // Redhat style:
    dirList.append("/usr/share/" + appDirName);

    // SuSE style:
    dirList.append("/usr/X11R6/" + appDirName);

    dirList.append("/usr/X11R6/share/" + appDirName);
    dirList.append(getHomeDir() + "/." + appDirName);

    // Local directory:
    dirList.append(".");
    //dirList.append(getCurrentDir());

	// Debian Doc:
	/usr/share/doc/qcad/html/en/
	*/

	RS_StringList dirList = getDirectoryList(subDirectory);

    RS_StringList fileList;
    RS_String path;
    RS_Dir dir;

    for (RS_StringList::Iterator it = dirList.begin();
            it!=dirList.end();
            ++it ) {

        //path = RS_String(*it) + "/" + subDirectory;
        path = RS_String(*it);
        dir = RS_Dir(path);

        if (dir.exists() && dir.isReadable()) {
            RS_StringList files = dir.entryList("*." + fileExtension);
            for (RS_StringList::Iterator it2 = files.begin();
                    it2!=files.end();
                    it2++) {

                fileList += path + "/" + (*it2);
            }
        }
    }

    return fileList;
}



/**
 * @return List of all directories in subdirectory 'subDirectory' in
 * all possible QCad directories.
 */
RS_StringList RS_System::getDirectoryList(const RS_String& subDirectory) {
    RS_StringList dirList;

    // Redhat style:
    dirList.append("/usr/share/" + appDirName + "/" + subDirectory);

    // SuSE style:
    dirList.append("/usr/X11R6/" + appDirName + "/" + subDirectory);

    dirList.append("/usr/X11R6/share/" + appDirName + "/" + subDirectory);
    dirList.append(getHomeDir() + "/." + appDirName + "/" + subDirectory);
	
    // Local directory:
    dirList.append("./" + subDirectory);

	// Individual directories:
   	RS_SETTINGS->beginGroup("/Paths");
	if (subDirectory=="fonts") {
	   	dirList += RS_StringList::split(RS_RegExp("[;]"), 
			RS_SETTINGS->readEntry("/Fonts", ""));
	} else if (subDirectory=="patterns") {
	   	dirList += RS_StringList::split(RS_RegExp("[;]"),
			RS_SETTINGS->readEntry("/Patterns", ""));
	} else if (subDirectory.startsWith("scripts")) {
	   	dirList += RS_StringList::split(RS_RegExp("[;]"),
			RS_SETTINGS->readEntry("/Scripts", ""));
	} else if (subDirectory.startsWith("library")) {
	   	dirList += RS_StringList::split(RS_RegExp("[;]"),
			RS_SETTINGS->readEntry("/Library", ""));
	} else if (subDirectory.startsWith("po")) {
	   	dirList += RS_StringList::split(RS_RegExp("[;]"),
			RS_SETTINGS->readEntry("/Translations", ""));
	}
   	RS_SETTINGS->endGroup();

	//std::cout << "Paths:\n";
	//for (RS_StringList::Iterator it = dirList.begin(); 
	//	it!=dirList.end(); ++it ) {
	//	std::cout << (*it) << "\n";
	//}

	return dirList;
}



/**
 * Converts a language string to a symbol (e.g. Deutsch or German to 'de').
 * 
 * Supported languages: http://ftp.ics.uci.edu/pub/ietf/http/related/iso639.txt
 */
RS_String RS_System::languageToSymbol(const RS_String& lang) {
    RS_String l = lang.lower();

    if (l=="afar") {
        return "aa";
    } else if (l=="abkhazian") {
        return "ab";
    } else if (l=="afrikaans") {
        return "af";
    } else if (l=="amharic") {
        return "am";
    } else if (l=="arabic") {
        return "ar";
    } else if (l=="assamese") {
        return "as";
    } else if (l=="aymara") {
        return "ay";
    } else if (l=="azerbaijani") {
        return "az";
    } else if (l=="bashkir") {
        return "ba";
    } else if (l=="byelorussian") {
        return "be";
    } else if (l=="bulgarian") {
        return "bg";
    } else if (l=="bihari") {
        return "bh";
    } else if (l=="bislama") {
        return "bi";
    } else if (l=="bengali" || l=="bangla") {
        return "bn";
    } else if (l=="tibetan") {
        return "bo";
    } else if (l=="breton") {
        return "br";
    } else if (l=="catalan") {
        return "ca";
    } else if (l=="corsican") {
        return "co";
    } else if (l=="czech") {
        return "cs";
    } else if (l=="welsh") {
        return "cy";
    } else if (l=="danish") {
        return "da";
    } else if (l=="german" || l=="deutsch") {
        return "de";
    } else if (l=="bhutani") {
        return "dz";
    } else if (l=="greek") {
        return "el";
    } else if (l=="english") {
        return "en";
    } else if (l=="esperanto") {
        return "eo";
    } else if (l=="spanish") {
        return "es";
    } else if (l=="estonian") {
        return "et";
    } else if (l=="basque") {
        return "eu";
    } else if (l=="persian") {
        return "fa";
    } else if (l=="finnish") {
        return "fi";
    } else if (l=="fiji") {
        return "fj";
    } else if (l=="faroese") {
        return "fo";
    } else if (l=="french" || l=="francais") {
        return "fr";
    } else if (l=="frisian") {
        return "fy";
    } else if (l=="irish") {
        return "ga";
    } else if (l=="scots gaelic" || l=="gaelic") {
        return "gd";
    } else if (l=="galician") {
        return "gl";
    } else if (l=="guarani") {
        return "gn";
    } else if (l=="gujarati") {
        return "gu";
    } else if (l=="hausa") {
        return "ha";
    } else if (l=="hebrew") {
        return "he";
    } else if (l=="hindi") {
        return "hi";
    } else if (l=="croatian") {
        return "hr";
    } else if (l=="hungarian") {
        return "hu";
    } else if (l=="armenian") {
        return "hy";
    } else if (l=="interlingua") {
        return "ia";
    } else if (l=="indonesian") {
        return "id";
    } else if (l=="interlingue") {
        return "ie";
    } else if (l=="inupiak") {
        return "ik";
    } else if (l=="icelandic") {
        return "is";
    } else if (l=="italian") {
        return "it";
    } else if (l=="inuktitut") {
        return "iu";
    } else if (l=="japanese") {
        return "ja";
    } else if (l=="javanese") {
        return "jw";
    } else if (l=="georgian") {
        return "ka";
    } else if (l=="kazakh") {
        return "kk";
    } else if (l=="greenlandic") {
        return "kl";
    } else if (l=="cambodian") {
        return "km";
    } else if (l=="kannada") {
        return "kn";
    } else if (l=="korean") {
        return "ko";
    } else if (l=="kashmiri") {
        return "ks";
    } else if (l=="kurdish") {
        return "ku";
    } else if (l=="kirghiz") {
        return "ky";
    } else if (l=="latin") {
        return "la";
    } else if (l=="lingala") {
        return "ln";
    } else if (l=="laothian") {
        return "lo";
    } else if (l=="lithuanian") {
        return "lt";
    } else if (l=="latvian" || l=="lettish") {
        return "lv";
    } else if (l=="malagasy") {
        return "mg";
    } else if (l=="maori") {
        return "mi";
    } else if (l=="macedonian") {
        return "mk";
    } else if (l=="malayalam") {
        return "ml";
    } else if (l=="mongolian") {
        return "mn";
    } else if (l=="moldavian") {
        return "mo";
    } else if (l=="marathi") {
        return "mr";
    } else if (l=="malay") {
        return "ms";
    } else if (l=="maltese") {
        return "mt";
    } else if (l=="burmese") {
        return "my";
    } else if (l=="nauru") {
        return "na";
    } else if (l=="nepali") {
        return "ne";
    } else if (l=="dutch") {
        return "nl";
    } else if (l=="norwegian") {
        return "no";
    } else if (l=="occitan") {
        return "oc";
    } else if (l=="afan" || l=="oromo" || l=="afan oromo") {
        return "om";
    } else if (l=="oriya") {
        return "or";
    } else if (l=="punjabi") {
        return "pa";
    } else if (l=="polish") {
        return "pl";
    } else if (l=="pashto" || l=="pushto") {
        return "ps";
    } else if (l=="portuguese") {
        return "pt";
    } else if (l=="quechua") {
        return "qu";
    } else if (l=="rhaeto-romance") {
        return "rm";
    } else if (l=="kirundi") {
        return "rn";
    } else if (l=="romanian") {
        return "ro";
    } else if (l=="russian") {
        return "ru";
    } else if (l=="kinyarwanda") {
        return "rw";
    } else if (l=="sanskrit") {
        return "sa";
    } else if (l=="sindhi") {
        return "sd";
    } else if (l=="sangho") {
        return "sg";
    } else if (l=="serbo-Croatian") {
        return "sh";
    } else if (l=="sinhalese") {
        return "si";
    } else if (l=="slovak") {
        return "sk";
    } else if (l=="slovenian") {
        return "sl";
    } else if (l=="samoan") {
        return "sm";
    } else if (l=="shona") {
        return "sn";
    } else if (l=="somali") {
        return "so";
    } else if (l=="albanian") {
        return "sq";
    } else if (l=="serbian") {
        return "sr";
    } else if (l=="siswati") {
        return "ss";
    } else if (l=="sesotho") {
        return "st";
    } else if (l=="sundanese") {
        return "su";
    } else if (l=="swedish") {
        return "sv";
    } else if (l=="swahili") {
        return "sw";
    } else if (l=="tamil") {
        return "ta";
    } else if (l=="telugu") {
        return "te";
    } else if (l=="tajik") {
        return "tg";
    } else if (l=="thai") {
        return "th";
    } else if (l=="tigrinya") {
        return "ti";
    } else if (l=="turkmen") {
        return "tk";
    } else if (l=="tagalog") {
        return "tl";
    } else if (l=="setswana") {
        return "tn";
    } else if (l=="tonga") {
        return "to";
    } else if (l=="turkish") {
        return "tr";
    } else if (l=="tsonga") {
        return "ts";
    } else if (l=="tatar") {
        return "tt";
    } else if (l=="twi") {
        return "tw";
    } else if (l=="uighur") {
        return "ug";
    } else if (l=="ukrainian") {
        return "uk";
    } else if (l=="urdu") {
        return "ur";
    } else if (l=="uzbek") {
        return "uz";
    } else if (l=="vietnamese") {
        return "vi";
    } else if (l=="volapuk") {
        return "vo";
    } else if (l=="wolof") {
        return "wo";
    } else if (l=="xhosa") {
        return "xh";
    } else if (l=="yiddish") {
        return "yi";
    } else if (l=="yoruba") {
        return "yo";
    } else if (l=="zhuang") {
        return "za";
    } else if (l=="chinese") {
        return "zh";
    } else if (l=="zulu") {
        return "zu";
    }

    return "";
}



/**
 * Converst a language two-letter-code into a readable string 
 * (e.g. 'de' to Deutsch)
 */
RS_String RS_System::symbolToLanguage(const RS_String& symb) {
    RS_String l = symb.lower();

    if (l=="aa") {
        return "Afar";
    } else if (l=="ab") {
        return "Abkhazian";
    } else if (l=="af") {
        return "Afrikaans";
    } else if (l=="am") {
        return "Amharic";
    } else if (l=="ar") {
        return "Arabic";
    } else if (l=="as") {
        return "Assamese";
    } else if (l=="ay") {
        return "Aymara";
    } else if (l=="az") {
        return "Azerbaijani";
    } else if (l=="ba") {
        return "Bashkir";
    } else if (l=="be") {
        return "Byelorussian";
    } else if (l=="bg") {
        return "Bulgarian";
    } else if (l=="bh") {
        return "Bihari";
    } else if (l=="bi") {
        return "Bislama";
    } else if (l=="bn") {
        return "Bengali";
    } else if (l=="bo") {
        return "Tibetan";
    } else if (l=="br") {
        return "Breton";
    } else if (l=="ca") {
        return "Catalan";
    } else if (l=="co") {
        return "Corsican";
    } else if (l=="cs") {
        return "Czech";
    } else if (l=="cy") {
        return "Welsh";
    } else if (l=="da") {
        return "Danish";
    } else if (l=="de") {
        return "German";
    } else if (l=="dz") {
        return "Bhutani";
    } else if (l=="el") {
        return "Greek";
    } else if (l=="en") {
        return "English";
    } else if (l=="eo") {
        return "Esperanto";
    } else if (l=="es") {
        return "Spanish";
    } else if (l=="et") {
        return "Estonian";
    } else if (l=="eu") {
        return "Basque";
    } else if (l=="fa") {
        return "Persian";
    } else if (l=="fi") {
        return "Finnish";
    } else if (l=="fj") {
        return "Fiji";
    } else if (l=="fo") {
        return "Faroese";
    } else if (l=="fr") {
        return "French";
    } else if (l=="fy") {
        return "Frisian";
    } else if (l=="ga") {
        return "Irish";
    } else if (l=="gd") {
        return "Scots Gaelic";
    } else if (l=="gl") {
        return "Galician";
    } else if (l=="gn") {
        return "Guarani";
    } else if (l=="gu") {
        return "Gujarati";
    } else if (l=="ha") {
        return "Hausa";
    } else if (l=="he") {
        return "Hebrew";
    } else if (l=="hi") {
        return "Hindi";
    } else if (l=="hr") {
        return "Croatian";
    } else if (l=="hu") {
        return "Hungarian";
    } else if (l=="hy") {
        return "Armenian";
    } else if (l=="ia") {
        return "Interlingua";
    } else if (l=="id") {
        return "Indonesian";
    } else if (l=="ie") {
        return "Interlingue";
    } else if (l=="ik") {
        return "Inupiak";
    } else if (l=="is") {
        return "Icelandic";
    } else if (l=="it") {
        return "Italian";
    } else if (l=="iu") {
        return "Inuktitut";
    } else if (l=="ja") {
        return "Japanese";
    } else if (l=="jw") {
        return "Javanese";
    } else if (l=="ka") {
        return "Georgian";
    } else if (l=="kk") {
        return "Kazakh";
    } else if (l=="kl") {
        return "Greenlandic";
    } else if (l=="km") {
        return "Cambodian";
    } else if (l=="kn") {
        return "Kannada";
    } else if (l=="ko") {
        return "Korean";
    } else if (l=="ks") {
        return "Kashmiri";
    } else if (l=="ku") {
        return "Kurdish";
    } else if (l=="ky") {
        return "Kirghiz";
    } else if (l=="la") {
        return "Latin";
    } else if (l=="ln") {
        return "Lingala";
    } else if (l=="lo") {
        return "Laothian";
    } else if (l=="lt") {
        return "Lithuanian";
    } else if (l=="lv") {
        return "Latvian";
    } else if (l=="mg") {
        return "Malagasy";
    } else if (l=="mi") {
        return "Maori";
    } else if (l=="mk") {
        return "Macedonian";
    } else if (l=="ml") {
        return "Malayalam";
    } else if (l=="mn") {
        return "Mongolian";
    } else if (l=="mo") {
        return "Moldavian";
    } else if (l=="mr") {
        return "Marathi";
    } else if (l=="ms") {
        return "Malay";
    } else if (l=="mt") {
        return "Maltese";
    } else if (l=="my") {
        return "Burmese";
    } else if (l=="na") {
        return "Nauru";
    } else if (l=="ne") {
        return "Nepali";
    } else if (l=="nl") {
        return "Dutch";
    } else if (l=="no") {
        return "Norwegian";
    } else if (l=="oc") {
        return "Occitan";
    } else if (l=="om") {
        return "Afan Oromo";
    } else if (l=="or") {
        return "Oriya";
    } else if (l=="pa") {
        return "Punjabi";
    } else if (l=="pl") {
        return "Polish";
    } else if (l=="ps") {
        return "Pashto";
    } else if (l=="pt") {
        return "Portuguese";
    } else if (l=="qu") {
        return "Quechua";
    } else if (l=="rm") {
        return "Rhaeto-Romance";
    } else if (l=="rn") {
        return "Kirundi";
    } else if (l=="ro") {
        return "Romanian";
    } else if (l=="ru") {
        return "Russian";
    } else if (l=="rw") {
        return "Kinyarwanda";
    } else if (l=="sa") {
        return "Sanskrit";
    } else if (l=="sd") {
        return "Sindhi";
    } else if (l=="sg") {
        return "Sangho";
    } else if (l=="sh") {
        return "Serbo-croatian";
    } else if (l=="si") {
        return "Sinhalese";
    } else if (l=="sk") {
        return "Slovak";
    } else if (l=="sl") {
        return "Slovenian";
    } else if (l=="sm") {
        return "Samoan";
    } else if (l=="sn") {
        return "Shona";
    } else if (l=="so") {
        return "Somali";
    } else if (l=="sq") {
        return "Albanian";
    } else if (l=="sr") {
        return "Serbian";
    } else if (l=="ss") {
        return "Siswati";
    } else if (l=="st") {
        return "Sesotho";
    } else if (l=="su") {
        return "Sundanese";
    } else if (l=="sv") {
        return "Swedish";
    } else if (l=="sw") {
        return "Swahili";
    } else if (l=="ta") {
        return "Tamil";
    } else if (l=="te") {
        return "Telugu";
    } else if (l=="tg") {
        return "Tajik";
    } else if (l=="th") {
        return "Thai";
    } else if (l=="ti") {
        return "Tigrinya";
    } else if (l=="tk") {
        return "Turkmen";
    } else if (l=="tl") {
        return "Tagalog";
    } else if (l=="tn") {
        return "Setswana";
    } else if (l=="to") {
        return "Tonga";
    } else if (l=="tr") {
        return "Turkish";
    } else if (l=="ts") {
        return "Tsonga";
    } else if (l=="tt") {
        return "Tatar";
    } else if (l=="tw") {
        return "Twi";
    } else if (l=="ug") {
        return "Uighur";
    } else if (l=="uk") {
        return "Ukrainian";
    } else if (l=="ur") {
        return "Urdu";
    } else if (l=="uz") {
        return "Uzbek";
    } else if (l=="vi") {
        return "Vietnamese";
    } else if (l=="vo") {
        return "Volapuk";
    } else if (l=="wo") {
        return "Wolof";
    } else if (l=="xh") {
        return "Xhosa";
    } else if (l=="yi") {
        return "Yiddish";
    } else if (l=="yo") {
        return "Yoruba";
    } else if (l=="za") {
        return "Zhuang";
    } else if (l=="zh") {
        return "Chinese";
    } else if (l=="zu") {
        return "Zulu";
    }

    return "";
}


/**
 * Testing
 */
bool RS_System::test() {
    RS_System::instance()->init("QCad test", "0.0.0", "qcad");
    RS_StringList l = RS_System::instance()->getFontList();

    //std::cout << "Font list: \n";

    for (RS_StringList::Iterator it = l.begin();
            it!=l.end();
            ++it ) {

        //std::cout << "Font: " << (*it).latin1() << "\n";
    }

    return true;
}



// EOF
